package it.arnaldo.unibs.tamagolem;

import java.util.*;


public class ElementGraph {
    private final Set<GraphElement> elements;

    public ElementGraph() {
        this.elements = new HashSet<>();
    }
    public void addElement(GraphElement element) {
        elements.add(element);
    }

    public int getValue(Element fromElement, Element toElement) {
        for (GraphElement graphElement : elements) {
            if (graphElement.getElement().equals(fromElement)) {
                return graphElement.getLinkValue(toElement);
            }
        }
        return 0;
    }

    public Set<GraphElement> getElements() {
        return elements;
    }

    public void printGraph() {
        System.out.println("From element is the row name, to element is the column name");
        List<Element> elements = getElementsNames(); // lista ordinata degli elementi
        int maxNameLen = 0;
        for (Element element : elements) {
            if (element.name().length() > maxNameLen) {
                maxNameLen = element.name().length();
            }
        }

        maxNameLen = maxNameLen + 2;

        // Stampa intestazione colonna
        System.out.print(String.format("%" + maxNameLen + "s", ""));
        for (Element to : elements) {
            System.out.print(String.format("%" + maxNameLen + "s", to));
        }
        System.out.println();
        System.out.println("-".repeat(10 + maxNameLen * elements.size()));

        // Stampa ogni riga
        for (Element from : elements) {
            System.out.print(String.format("%" + maxNameLen + "s", from));
            for (Element to : elements) {
                int value = getInteractionBetween(from, to);
                System.out.print(String.format("%" + maxNameLen + "s", value));
            }
            System.out.println();
        }


    }

    public int getTotalStrength(){
        int totalStrength = 0;

        for (GraphElement element : elements) {
            totalStrength += element.getTotalStrength();
        }

        return totalStrength;
    }

    public List<Element> getElementsNames(){
        List<Element> elements = new ArrayList<>();
        for (GraphElement el : this.elements ){
            elements.add(el.getElement());
        }
        return elements;
    }

    public int getInteractionBetween(Element fromElement, Element toElement) {
        for (GraphElement el : elements) {
            if (el.getElement().equals(fromElement)){
                for (ElementLink link : el.getLinks()) {
                    if (link.getElement().equals(toElement)){
                        return link.getValue();
                    }
                }
            }
        }

        return 0;
    }


}
